####################################################################
# RenderMan Shading Language Tokenizer
#
# Copyright (C) 2002, Matthias Baas (baas@ira.uka.de)
#
# http://cgkit.sourceforge.net
#
# You may distribute under the terms of the BSD license, as
# specified in the file license.txt.
####################################################################

import re

WHITESPACE = 0
NAME       = 1
NUMBER     = 2
STRING     = 3
NEWLINE    = 4
OPERATOR   = 5
CHARACTER  = 6
TYPE       = 7

IF_TOKEN    = 8
THEN_TOKEN  = 9
ELSE_TOKEN  = 10
RETURN_TOKEN = 9

# tokenize
def tokenize(readline, tokeater):

    regs =  ( (WHITESPACE, re.compile(r"[ ]+")),
              (TYPE,       re.compile(r"float|point|vector|normal|matrix|color")),
              (NAME,       re.compile(r"[A-Za-z_][A-Za-z_0-9]*")),
              (NUMBER,     re.compile(r"[0-9]+(\.[0-9]+)?(E(\+|-)?[0-9]+)?")),
              (STRING,     re.compile(r"\"[^\"]*\"")),
              (OPERATOR,   re.compile(r"\+|-|!|\.|\*|/|\^|<|>|<=|>=|==|!=|&&|\|\||\?|:|=|\(|\)")),
              (NEWLINE,    re.compile(r"\n"))
            )
              
    while 1:
        # Nchste Zeile lesen
        s=readline()
        # Keine Zeile mehr vorhanden, dann fertig
        if s=="":
            break

        # Kommentare ignorieren
        if s[0]=="#":
            continue

        # Zeile s in Tokens zerlegen...
        while s!="":
            unmatched=1
            # Regulre Ausdrcke der Reihe nach testen...
            for r in regs:
                m=r[1].match(s)
                # Passt er? dann ist Token gefunden
                if m!=None:
                    tok = s[m.start(): m.end()]
                    s   = s[m.end():]
                    tokeater(r[0],tok)
                    unmatched=0
                    continue

            # Hat kein regulrer Ausdruck gepasst? dann Einzelzeichen
            # als Character-Token ausgeben
            if unmatched:
                tok = s[0]
                tokeater(CHARACTER, tok)
                s=s[1:]
            
            

def _tokeater(type, s):
    if type==WHITESPACE or type==NEWLINE:
        return
    print "Token:",type,s

######################################################################

if __name__=="__main__":

    f=open("test.sl")
#    f=open("plank_cpp.sl")
    tokenize(f.readline, _tokeater)
    
