## RIB exporter for Curious Labs Poser 5
## (c) 2003 Stefan Werner stew@keindesign.de
## requires Poser 5 SR3 on Windows with cgkit and 3Delight installed


from ri import *
from cgtypes import *
import poser
import os
import math

def radians(degrees):
    return degrees / 360 * math.pi


def cross (A, B):
	return (A[1]*B[2] - A[2]*B[1], A[2]*B[0] - A[0]*B[2], A[0]*B[1] - A[1]*B[0])
def _absVec ((x, y, z)):
	return math.sqrt (x*x + y*y + z*z)
def norm ((x, y, z)):
	length = _absVec ((x, y, z))
	if (length == 0):
		return ((0,0,0))
	return ((x/length, y/length, z/length))
def sub (B, A):
	return (B[0]-A[0], B[1]-A[1], B[2]-A[2])
def add (A, B):
	return (B[0]+A[0], B[1]+A[1], B[2]+A[2])

basefilename = "d:\\test"
renderer = "3delight"
#renderer = "aqsis"
POINTS = 0
SUBDIV = 1
TEXTURES = 1
ANIM = 0
SHADOWS = 1

import tkFileDialog
import Tkinter


class App:
    def calcNormals(caller, faces, verts, sets):
        fnorms = []
        vnorms = []   
        for v in verts:
            vnorms.append((0,0,0))
        for f in faces:
            start = f.Start()
            a = verts[sets[start]]
            #a = (a.X(), a.Y(), a.Z())
            b = verts[sets[start+1]]
            #b = (b.X(), b.Y(), b.Z())
            c = verts[sets[start+2]]
            #c = (c.X(), c.Y(), c.Z())
            v1 = sub(a, b)
            v2 = sub(a, c)
            n = norm(cross(v1, v2))
            fnorms.append( norm(cross(v1, v2)))
            for i in range(f.NumVertices()):
                vnorms[sets[start+i]] = add(vnorms[sets[start+i]],n)
        i = 0
        #for f in faces:
            #start = f.Start()
            #if (f.NumVertices() == 3):
            #    if ((sets[start]==i) or (sets[start+1]==i) or (sets[start+2]==i)):
            #        n = add(n,fnorms[j])
            #        avg = avg+1
            #elif (f.NumVertices() == 4):
            #   if ((sets[start]==i) or (sets[start+1]==i) or (sets[start+2]==i) or (sets[start+3]==i)):
            #       n = add(n,fnorms[j])
            #       avg = avg+1
            #j = j + 1
        for n in vnorms:
            n = norm(n)
        #vnorms.append( norm(n))
        i = i + 1
        return vnorms



    def __init__(self, master, textMessage):
        self.master = master
        basefilename = tkFileDialog.asksaveasfilename()
        if (basefilename == ""):
            raise "no file selected"


        scene = poser.Scene()

        if (ANIM):
            frames = range(scene.NumFrames())
        else:
            frames = [scene.Frame()]

        for frame in frames:
            scene.SetFrame(frame)
            scene.DrawAll()
            scene.ProcessSomeEvents(1)
            filename = basefilename + str(frame) + ".rib"

            RiBegin(filename)
            res = scene.OutputRes()
            RiFormat (res[0], res[1], 1)
            RiDisplay (basefilename + str(frame) + ".tif", "tiff", "rgba")
            RiPixelFilter ("gaussian",2,2)
            RiPixelSamples (3,3)
            RiOption ("trace", "integer maxdepth", [2])

            RiClipping (0.1, 1000.0)
            cam = scene.CurrentCamera()
            if (cam.Parameter('Focal').Value() != 0):
                angle = 2.0*math.atan(25.4/(cam.Parameter('Focal').Value() * 2.0))/math.pi*180
                RiProjection ("perspective", "fov", angle)
            else:
                RiProjection ("orthographic")

            m = mat4 (cam.WorldMatrix()).scale((1,1,-1,1))
            m = m.inverse()
            RiTransform (m)

            RiWorldBegin()
            RiDeclare ("shadows", "string")
            RiDeclare ("transmission", "string[]")
            RiDeclare ("trace", "float[]")
            RiDeclare ("shadinggroup", "string")

            #declare our poser shader

            RiDeclare ("ambient", "uniform float")
            RiDeclare ("diffusion", "uniform float")
            RiDeclare ("specular", "uniform float")
            RiDeclare ("roughness", "uniform float")
            RiDeclare ("ambientTexture", "uniform string")
            RiDeclare ("ambientColor", "uniform color")
            RiDeclare ("diffuseTexture", "uniform string")
            RiDeclare ("diffuseColor", "uniform color")
            RiDeclare ("specularTexture", "uniform string")
            RiDeclare ("specularColor", "uniform color")
            RiDeclare ("transTexture", "uniform string")
            RiDeclare ("transparency", "uniform float")
            RiDeclare ("bumpTexture", "uniform string")
            RiDeclare ("bumpHeight", "uniform float")
            RiDeclare ("shadingrate", "uniform float")
            RiDeclare ("nsamples", "uniform float")
            RiDeclare ("maxerror", "uniform float")

            lights = scene.Lights()
            n = 0
            RiAttributeBegin()
            for light in lights:
                n = n + 1
                col  = (light.Parameter('Red').Value(), light.Parameter('Green').Value(), light.Parameter('Blue').Value())
                int = light.Parameter('Intensity').Value()
                m = mat4 (light.WorldMatrix())
                RiTransformBegin()
                RiTransform (m.scale((1,1,-1)))
                if (SHADOWS):
                    if  light.RayTraceShadows():
                        RiAttribute ("light", "shadows", "on")
                    else:
                        RiAttribute ("light", "shadows", "off")
                
                if (light.LightType() == 0):
                    RiLightSource(RI_DISTANTLIGHT, "lightcolor", col, "intensity", int)
                else:
                    angle = light.Parameter("Angle End").Value()
                    delta = angle - light.Parameter("Angle Start").Value()
                    RiLightSource(RI_SPOTLIGHT, "lightcolor", col, "intensity", int, \
                                  "coneangle", radians(angle), "conedeltaangle", radians(delta))
                RiIlluminate (n, RI_TRUE)
                RiTransformEnd()

            RiTransformBegin()
            RiIdentity()
            RiOrientation("rh")
            RiTransformEnd()

            RiShadingRate(1)
            RiSides (1)
            RiShadingInterpolation("smooth")
            RiAttribute ("visibility", "trace", [1])
            #RiAttribute ("irradiance", "nsamples", [128])
            #RiAttribute ("irradiance", "shadingrate", [4])
            #RiAttribute ("irradiance", "maxerror", [0.2])
            RiAttribute ("trace", "float bias", [.001])

            actors = scene.Actors()
            for actor in actors:
                if ( (actor.IsProp() == 0) and (actor.ItsFigure() == None) ):
                    continue
                if ( actor.IsDeformer() or actor.IsBase() or actor.IsZone() ):
                   continue
                if (actor.OnOff() == 0):
                   continue
                geom = actor.Geometry()
                if (geom == None):
                   continue
                pverts = geom.WorldVertices()
                if (pverts == None):
                   continue
                sets = geom.Sets()
                polys = geom.Polygons()
                verts = []
                for vert in pverts:
                    verts.append((vert.X(), vert.Y(), vert.Z()))
                shadingRate = actor.ShadingRate()
                          
                if (shadingRate == 8.0):
                    RiAttributeBegin()
                    RiShadingRate(8)
                    RiColor([1,1,1])
                    RiSurface("srf_fur")
                    if (SHADOWS):
                        if (renderer == "3delight"):
                            RiAttribute ("visibility", "trace", [1])
                            RiAttribute ("visibility", "transmission", ["opaque"])
                    for poly in polys:
                        num = poly.NumVertices()
                        if (num < 3):
                           continue
                        vertlist = []
                        start = poly.Start()
                        for i in range(num):
                            vert = verts[sets[start + i]]
                            vertlist.append(vert)
                        RiCurves (RI_CUBIC,[num], RI_NONPERIODIC, RI_P, vertlist, RI_CONSTANTWIDTH, [0.00025])
                    RiAttributeEnd()
                    continue
           
                RiAttributeBegin()
                RiAttribute ("identifier", "shadinggroup", [actor.Name()])
                if (shadingRate > 1):
                    RiShadingRate(shadingRate)
                else:
                    RiShadingRate(1)

                texsets = geom.TexSets()
                texpolys = geom.TexPolygons()
                texverts = geom.TexVertices()
                norms = self.calcNormals(polys, verts, sets)
                numvertlist = []
                vertlist = []
                normlist = []
                stlist = []
                matlist = []
                i = 0
                prevMatIndex = -1

                if (TEXTURES):
                    mats = geom.Materials()
                    for mat in geom.Materials():
                        Cd = mat.DiffuseColor()
                        Cs = mat.SpecularColor()
                        Ks = mat.SpecularColor()
                        Ns = 1/mat.Ns()
                        Op = 1-mat.TransparencyMax()
                        texname = mat.TextureMapFileName()
                        if (texname != None):
                            (path, file) = os.path.split(texname)
                            (name, ext) = os.path.splitext(file)
                            (newpath, file) = os.path.split(basefilename)
                            newpath = newpath + "/"
                            if (renderer == "3delight"):
                                if (os.path.exists(newpath + name + ".tdl")):
                                    texname = newpath + name + ".tdl"
                                else:
                                    command = os.environ["DELIGHT"] + "\\bin\\tdlmake \"" + texname + "\" \"" + newpath + name + ".tdl" + "\""
                                    os.system(command)
                                    texname = newpath + name + ".tdl"
                            if (renderer == "bmrt"):
                                texname = string.replace (texname, '\\', '\\\\')
                        else:
                            texname = ""
                        transname = mat.TransparencyMapFileName()
                        if (transname != None):
                            (path, file) = os.path.split(transname)
                            (name, ext) = os.path.splitext(file)
                            (newpath, file) = os.path.split(basefilename)
                            newpath = newpath + "/"
                            if (renderer == "3delight"):
                                if (os.path.exists(newpath+ name + ".tdl")):
                                    transname = newpath + name + ".tdl"
                                else:
                                    command = os.environ["DELIGHT"] + "\\bin\\tdlmake \"" + transname + "\" \"" + newpath + name + ".tdl" + "\""
                                    os.system(command)
                                    transname = newpath + name + ".tdl"
                            if (renderer == "bmrt"):
                                transname = string.replace (transname, '\\', '\\\\')
                        else:
                            transname = ""
                        bumpname = mat.BumpMapFileName()
                        if (bumpname != None):
                            (path, file) = os.path.split(bumpname)
                            (name, ext) = os.path.splitext(file)
                            if (ext == ".bum"):
                                bumpname = ""
                            (newpath, file) = os.path.split(basefilename)
                            newpath = newpath + "/"
                            if (renderer == "3delight"):
                                if (bumpname != ""):
                                    if (os.path.exists(newpath + name + ".tdl")):
                                        bumpname = newpath + name + ".tdl"
                                    else:
                                        command = os.environ["DELIGHT"] + "\\bin\\tdlmake \"" + bumpname + "\" \"" + newpath + name + ".tdl" + "\""
                                        os.system(command)
                                        bumpname = newpath + name + ".tdl"
                                if (renderer == "bmrt"):
                                    bumpname = string.replace (tbumpname, '\\', '\\\\')
                        else:
                            bumpname = ""
                        bumpstrength = mat.BumpStrength()
                        matlist.append((Cd, Cs, Ks, Ns, texname, Op, transname, bumpname, bumpstrength))
                if (POINTS):
                    matIndex = polys[0].MaterialIndex()
                    if (matIndex != prevMatIndex):
                        if (matlist[matIndex][6] == ""):
                            RiAttribute ("visibility", "transmission", ["opaque"])
                        else:
                            RiAttribute ("visibility", "transmission", ["shader"])
                        #RiAttribute ("visibility", "transmission", ["opaque"])
                        RiSurface ("poserSurface", "diffuseTexture", [matlist[matIndex][4]], "diffuseColor", [matlist[matIndex][0]], "transparency", [matlist[matIndex][5]], \
                                   "transTexture", [matlist[matIndex][6]], "specular", [0], "bumpTexture", [matlist[matIndex][7]],\
                                   "bumpHeight", [bumpstrength], "roughness", [matlist[matIndex][3]], "specularColor", [matlist[matIndex][1]])
                    for poly in polys:
                        num = poly.NumVertices()
                        numvertlist.append(num)
                        start = poly.Start()
                        for i in range(num):
                            vert = verts[sets[start + i]]
                            #vertlist.append((vert.X(), vert.Y(), vert.Z()))
                            #norm = norms[sets[start + i]]
                            #normlist.append((norm.X(), norm.Y(), norm.Z()))
                            uv = texverts[texsets[start + i]]
                            stlist.append((uv.U(), 1-uv.V()))
                    vertlist = []
                    for vert in verts:
                    	vertlist.append((vert.X(), vert.Y(), vert.Z()))

                    if (actor.SmoothPolys() and SUBDIV):
                        RiSubdivisionMesh("catmull-clark", numvertlist, sets,[RI_INTERPOLATEBOUNDARY],[0,0],[], [], RI_P, vertlist, RI_ST, stlist)
                    else:
                         RiPointsPolygons(numvertlist, sets,  RI_P, vertlist, RI_ST, stlist)
                else:
                    for poly in polys:
                        #scene.ProcessSomeEvents(1)
                        matIndex = poly.MaterialIndex()
                        if (matIndex != prevMatIndex):
                            if (matlist[matIndex][6] == ""):
                                RiAttribute ("visibility", "transmission", ["opaque"])
                            else:
                                RiAttribute ("visibility", "transmission", ["shader"])
                            RiSurface ("poserSurface", "diffuseTexture", [matlist[matIndex][4]], "diffuseColor", [matlist[matIndex][0]], "transparency", [matlist[matIndex][5]], \
                                       "transTexture", [matlist[matIndex][6]], "specular", [0], "bumpTexture", [matlist[matIndex][7]],\
                                       "bumpHeight", [bumpstrength], "roughness", [matlist[matIndex][3]], "specularColor", [matlist[matIndex][1]])
                            #RiOpacity (matlist[matIndex][5])
                            #RiColor (matlist[matIndex][0])
                            #if (matlist[matIndex][4] != None):
                            #    RiSurface(RI_PAINTEDPLASTIC,  RI_TEXTURENAME, matlist[matIndex][4], "Kd", [0.812372],\
                            #"specularcolor", [matlist[matIndex][1]], RI_ROUGHNESS, matlist[matIndex][3])
                            #else:
                            #    RiSurface(RI_PAINTEDPLASTIC, "Kd", [0.812372], "specularcolor", [matlist[matIndex][1]], RI_ROUGHNESS, matlist[matIndex][3])
                            prevMatIndex = matIndex
                        num = poly.NumVertices()
                        if (num < 3):
                            continue
                        vertlist = []
                        uvlist = []
                        normlist = []
                        stlist = []
                        start = poly.Start()
                        for i in range(num):
                            vert = verts[sets[start + i]]
                            #vertlist.append((vert.X(), vert.Y(), vert.Z()))
                            vertlist.append(vert)
                            uv = texverts[texsets[start + i]]
                            stlist.append((uv.U(), 1-uv.V()))
                            norm = norms[sets[start + i]]
                            normlist.append(norm)
                        RiPolygon (RI_P, vertlist, RI_N, normlist, RI_ST, stlist)
                        #RiPolygon (RI_P, vertlist, RI_ST, stlist)
                RiAttributeEnd()

            RiAttributeEnd()
            RiWorldEnd()
            RiEnd()

        #os.system("aqsis -fb " + filename) # Aqsis
        if (renderer == "bmrt"):
            os.system("rendrib -d " + filename) # BMRT
        if (renderer == "3delight"):
            args = [os.environ["DELIGHT"] + "\\bin\\renderdl.exe", "-D"]
            for i in frames:
                args.append ("\"" + basefilename + str(i) + ".rib\"")
            os.spawnv(os.P_NOWAIT, os.environ["DELIGHT"] + "\\bin\\renderdl.exe", args)
            #os.system("renderdl -d -stats " + filename) # 3Delight
        if (renderer == "aqsis"):
            os.system("aqsis -fb " + filename) # Aqsis

        
        self.master.destroy()
        self.master.quit()

root = Tkinter.Tk()
app = App(root, "")
root.mainloop()
